#include <SD.h>
#include <Wire.h> 
int I2C_eep = 0x50;  // EEPROM の I2C アドレス 
int I2C_lcd = 0x3E;  // LCD の I2C アドレス 
byte ERROR;
// #define I2C_eep 0x54  // EEPROM の I2C アドレス 

File myFile;
char filename[]="EEPROM.HEX";
// 21d0
// http://baticadila.dip.jp/arduino_013.html

// 1バイト読込み関数
byte eprReadByte(unsigned int addr) {
	byte len,ret,retry=3;
	
	while(retry>0){
		Wire.beginTransmission(I2C_eep);
		Wire.write(highByte(addr));
		Wire.write(lowByte(addr));
		ret=Wire.endTransmission();
		if(ret==0){
			len=Wire.requestFrom(I2C_eep, 1);
			if(len==1){
				ret = Wire.read();
				ERROR=0;
				retry=0;
			}else{
				lcdPrint("ERR I2C ReadRqst");
				delay(500);
				ERROR=1;
				retry--;
			}
		}else{
			lcdPrint("ERR I2C ReadTrns");
			delay(500);
			ERROR=1;
			retry--;
		}
	}
	return (ret);
}

void utf_del_uni(char *s){
	byte i=0;
	byte j=0;
	while(s[i]!='\0'){
		if((byte)s[i]==0xEF){
			if((byte)s[i+1]==0xBE) s[i+2] += 0x40;
			i+=2;
		}
		s[j]=s[i];
		i++;
		j++;
	}
	s[j]='\0';
}

void lcdOut(byte y,byte *lcd){
	byte data;
	byte i;
	if(y==0) data=0x80;
	else{
		data=0xC0;
		y=1;
	}
	Wire.beginTransmission(I2C_lcd);
	Wire.write(0x00);Wire.write(data);
	Wire.endTransmission();
	for(i=0;i<8;i++){
		if(lcd[i]==0x00) break;
		Wire.beginTransmission(I2C_lcd);
		Wire.write(0x40);Wire.write(lcd[i]);
		Wire.endTransmission();
	}
}

void lcdPrint(char *s){
	byte i,j;
	byte lcd[9];
	
	utf_del_uni(s);
	lcd[8]='\0';
	for(j=0;j<2;j++){
		for(i=0;i<8;i++){
			lcd[i]=(byte)s[i+8*j];
			if(lcd[i]==0x00)for(;i<8;i++) lcd[i]=' ';
		}
		lcdOut(j,lcd);
	}
}

void lcdSetup() {
	Wire.beginTransmission(I2C_lcd);
	Wire.write(0x00);Wire.write(0x39);Wire.endTransmission();
	delay(10);
	Wire.beginTransmission(I2C_lcd);
	Wire.write(0x00);Wire.write(0x11);Wire.endTransmission();
	delay(10);
	Wire.beginTransmission(I2C_lcd);
	Wire.write(0x00);Wire.write(0x70);Wire.endTransmission();
	delay(10);
	Wire.beginTransmission(I2C_lcd);
	Wire.write(0x00);Wire.write(0x56);Wire.endTransmission();
	delay(10);
	Wire.beginTransmission(I2C_lcd);
	Wire.write(0x00);Wire.write(0x6C);Wire.endTransmission();
	delay(200);
	Wire.beginTransmission(I2C_lcd);
	Wire.write(0x00);Wire.write(0x38);Wire.endTransmission();
	delay(10);
	Wire.beginTransmission(I2C_lcd);
	Wire.write(0x00);Wire.write(0x0C);Wire.endTransmission();
	lcdPrint("Hello!  I2C LCD ");
}

void byte2hex(char *s,byte in){
	if(in/16<10) s[0]=(byte)('0'+in/16);
	else s[0]=(byte)('A'+in/16-10);
	if(in%16<10) s[1]=(byte)('0'+in%16);
	else s[1]=(byte)('A'+in%16-10);
}

void setup(){
	unsigned int address=0;
	byte data,sum;
	char s[16];
	
	Serial.begin(9600);
	Wire.begin();
	lcdSetup();
	lcdPrint("Start SD");
	delay(1000);
	if (!SD.begin(4)) {
		lcdPrint("ERROR SD");
		delay(5000);
		return;
	}
	if(SD.exists(filename)){
		lcdPrint("Deleted HEX File");
		SD.remove(filename);
		delay(5000);
	}
	myFile = SD.open(filename, FILE_WRITE);
	lcdPrint("Read ROM");
	delay(1000);

	address=0;
	while(1){
		if(address%16==0){
			byte2hex(&s[0],I2C_eep);
			s[2]=' ';
			myFile.print(":10");
			sum=0x10;
			data=highByte(address);
			myFile.print(data/16,HEX);
			myFile.print(data%16,HEX);
			byte2hex(&s[3],data);
			sum+=data;
			data=lowByte(address);
			myFile.print(data/16,HEX);
			myFile.print(data%16,HEX);
			byte2hex(&s[5],data);
			sum+=data;
			myFile.print("00");
			s[7]=' ';
		}
		data=eprReadByte(address);
		if(ERROR){
			lcdPrint("ERROR");
			break;
		}
		sum+=data;
		myFile.print(data/16,HEX);
		myFile.print(data%16,HEX);
		if(address%16==15){
			byte2hex(&s[8],data);
			s[10]=' ';
			data = sum^0xffffffff;
			data++;
			myFile.print(data/16,HEX);
			myFile.println(data%16,HEX);
			data +=sum;
			byte2hex(&s[11],sum);
			s[13]=' ';
			byte2hex(&s[14],data);
			lcdPrint(s);
		}
		address++;
		if(address==0){
			if(I2C_eep == 0x50) I2C_eep = 0x54;
			else{
				myFile.println(":00000001FF");
				lcdPrint("SUCCESS");
				break;
			}
		}
	}
	myFile.close();
	delay(1000);
}

void loop()
{
	// nothing happens after setup
	while(1);
}


